package network.omisego.omgmerchant.pages.authorized.main.more.transaction

/*
 * OmiseGO
 *
 * Created by Phuchit Sirimongkolsathien on 29/8/2018 AD.
 * Copyright © 2017-2018 OmiseGO. All rights reserved.
 */

import android.content.Context
import co.omisego.omisego.model.Transaction
import co.omisego.omisego.model.TransactionSource
import co.omisego.omisego.model.pagination.Paginable
import network.omisego.omgmerchant.R
import network.omisego.omgmerchant.model.AmountFormat

class TransactionListTransformer(
    val context: Context
) {
    val TransactionSource.username: String
        get() = "${this.user?.email ?: this.user?.username}"
    val Transaction.isTopup: Boolean
        get() = this.from.accountId != null

    fun transformTransactionType(transaction: Transaction): String {
        return if (transaction.isTopup) {
            "\uE92F"
        } else {
            "\uE902"
        }
    }

    fun transformUsername(transaction: Transaction): String {
        return if (transaction.isTopup) {
            context.getString(
                R.string.transaction_list_info_name_id,
                transaction.to.username
            )
        } else {
            context.getString(
                R.string.transaction_list_info_name_id,
                transaction.from.username
            )
        }
    }

    fun transformDescription(transaction: Transaction): String {
        return if (transaction.error == null) {
            context.getString(R.string.transaction_list_info_success)
        } else {
            context.getString(R.string.transaction_list_info_failure)
        }
    }

    fun transformAmount(transaction: Transaction): String {
        val subunit = AmountFormat.Subunit(transaction.from.amount, transaction.from.token.subunitToUnit)
        val unit = subunit.toUnit()
        val amountText = context.getString(
            R.string.transaction_list_info_amount,
            unit.display(),
            transaction.from.token.symbol
        )
        return when (transaction.status) {
            Paginable.Transaction.TransactionStatus.CONFIRMED -> {
                if (transaction.to.accountId != null) {
                    "+ $amountText"
                } else {
                    "- $amountText"
                }
            }
            else -> {
                amountText
            }
        }
    }

    fun transformStatusIcon(transaction: Transaction): String {
        return if (transaction.error == null) {
            "\uE906"
        } else {
            "\uE90B"
        }
    }

    fun transformDate(transaction: Transaction): String {
        return context.getString(R.string.transaction_list_info_date_time, transaction.createdAt)
    }
}
